/*
 * in_game_screen.hpp
 *
 * InGameScreen
 * --- runs the game
 * Goes back to MenuScreen when done
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef IN_GAME_SCREEN_HPP
#define IN_GAME_SCREEN_HPP

#include "screen.hpp"
#include "coercri/gfx/window_listener.hpp"

class GameClient;
class GameServer;
class LocalDungeonView;
class MenuSelections;
class RemoteDungeonView;


class InGameScreen : public Screen, public Coercri::WindowListener {
public:
    InGameScreen(KnightsGame &kg, const MenuSelections &ms);
    virtual bool start(KnightsGame &, shared_ptr<Coercri::Window> win, gcn::Gui &gui);
    virtual ~InGameScreen();
    virtual unsigned int getUpdateInterval();
    virtual void update();

    virtual void onRepaint(Coercri::GfxContext &gfx_context, const Coercri::Region &rgn);
    virtual void onMouseDown(int x, int y, Coercri::MouseButton m);
    virtual void onRawKeyDown(Coercri::RawKey rk);
    
private:
    void serverSetup(const MenuSelections &msel);
    bool tryClientSetup();
        
private:
    KnightsGame &knights_game;
    shared_ptr<GameServer> game_server;
    shared_ptr<GameClient> game_client;
    shared_ptr<LocalDungeonView> ldv1, ldv2;
    shared_ptr<RemoteDungeonView> rdv1;
    bool first_update, sent_ready_yet, knights_spawned_yet;
    int prev_update_time;
    
    enum GameStage { SETUP_STAGE, HANDSHAKE_STAGE, INGAME_STAGE, GAMEOVER_STAGE };
    GameStage stage;

    shared_ptr<Coercri::Window> window;
};

#endif
