/*
 * menu.cpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "menu.hpp"
#include "menu_constraint.hpp"
#include "menu_selections.hpp"

void Menu::addSpaceAfter(const string &id)
{
    for (vector<MenuItem*>::iterator it = items.begin(); it != items.end(); ++it) {
        if ((*it)->getID() == id) {
            (*it)->setSpaceAfter(true);
            break;
        }
    }
}

bool MenuValue::applyConstraints(MenuSelections &msel) const
{
    bool result = false;
    for (vector<shared_ptr<MenuConstraint> >::const_iterator it = constraints.begin();
    it != constraints.end(); ++it) {
        if ((*it)->apply(msel)) result = true;
    }
    return result;
}

bool MenuValue::canSetTo(const MenuSelections &msel, const string &id, int v) const
{
    for (vector<shared_ptr<MenuConstraint> >::const_iterator it = constraints.begin();
    it != constraints.end(); ++it) {
        if (!(*it)->canSetTo(msel, id, v)) return false;
    }
    return true;
}

bool Menu::isSettingAllowed(const MenuSelections &msel, const string &id, int v) const
{
    for (vector<MenuItem *>::const_iterator item = items.begin(); item != items.end(); ++item) {
        const string &item_id = (*item)->getID();
        if (item_id != "quest") {    // allow them to set up a custom quest w/o having to deselect the current quest first.
            const int item_setting = msel.get(item_id);
            const MenuValue &item_value = (*item)->getValue(item_setting);
            if (!item_value.canSetTo(msel, id, v)) return false;
        }
    }
    return true;
}
